/*
	*
	* CArchiveModel.cpp - NewBreeze CArchiveModel Class
	*
*/

#include "carchivemodel.hpp"

#include <archive.h>
#include <archive_entry.h>
#include <cprime/filefunc.h>

static QMimeDatabase mimeDb;

/*
	*
	* CArchiveModel - For existing archives
	*
*/

CArchiveModel::CArchiveModel() : QAbstractItemModel() {
};

void CArchiveModel::loadArchive( QString path ) {

	/* Archive name and path */
	archiveName = path;

	/* Root node */
	growTree();
};

CArchiveModel::~CArchiveModel() {

};

int CArchiveModel::rowCount( const QModelIndex &parent ) const {

	CArchiveBranch *parentItem;
	if ( parent.column() > 0 )
		return 0;

	if ( not parent.isValid() )
		parentItem = tree;

	else
		parentItem = static_cast<CArchiveBranch*>( parent.internalPointer() );

	return parentItem->branchCount();
};

int CArchiveModel::columnCount( const QModelIndex & ) const {

	return 2;
};

Qt::ItemFlags CArchiveModel::flags( const QModelIndex &idx ) const {

	if ( not idx.isValid() )
		return Qt::NoItemFlags;

	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
};

QVariant CArchiveModel::data( const QModelIndex &index, int role ) const {

	if ( not index.isValid() )
		return QVariant();

	CArchiveBranch *node = static_cast<CArchiveBranch*>( index.internalPointer() );
	switch( role ) {

		case Qt::DisplayRole: {
			if ( index.column() == 0 )
				return node->name();

			else if ( index.column() == 1 )
				return node->size();

			return QVariant();
		}

		case Qt::DecorationRole: {
			if ( index.column() == 0 )
				return node->icon();

			return QVariant();
		}

		case Qt::TextAlignmentRole: {
			if ( index.column() == 0 )
				return ( 0x0001 | 0x0080 );

			else if ( index.column() == 1 )
				return ( 0x0002 | 0x0080 );

			else
				return Qt::AlignCenter;
		}

		case Qt::InitialSortOrderRole: {
			return Qt::AscendingOrder;
		}

		case Qt::AccessibleTextRole: {
			return node->name();
		}

		case Qt::ToolTipRole: {
			if ( index.column() == 0 )
				return node->data( Qt::UserRole + 1 );

			return QString();
		}

		case Qt::UserRole + 1: {

			return node->data( Qt::UserRole + 1 );
		}

		default: {

			return QVariant();
		}
	}
};

QVariant CArchiveModel::headerData( int section, Qt::Orientation orientation, int role ) const {

	if ( orientation != Qt::Horizontal )
		return QVariant();

	switch( section ) {
		case 0: {
			switch( role ) {
				case Qt::DisplayRole:
					return "Name";

				case Qt::TextAlignmentRole:
					return Qt::AlignCenter;

				default:
					return QVariant();
			}
		}

		case 1: {
			switch( role ) {
				case Qt::DisplayRole:
					return "Size";

				case Qt::TextAlignmentRole:
					return Qt::AlignCenter;

				default:
					return QVariant();
			}
		}

		default: {
			return QVariant();
		}
	}
};

QModelIndex CArchiveModel::index( int row, int column, const QModelIndex &parent ) const {

    if ( row < 0 or column < 0 )
        return QModelIndex();

    if ( ( row > rowCount( parent ) ) and ( column > columnCount( parent ) ) )
		return QModelIndex();

	CArchiveBranch *parentNode;
	if ( not parent.isValid() )
		parentNode = tree;

	else
		parentNode = (CArchiveBranch *)parent.internalPointer();

	CArchiveBranch *branch = parentNode->branch( row );
	if ( branch )
		return createIndex( row, column, branch );

	else
		return QModelIndex();
};

QModelIndex CArchiveModel::index( QString name, const QModelIndex &parent ) const {

	CArchiveBranch *parentNode;
	if ( not parent.isValid() )
		parentNode = tree;

	else
		parentNode = static_cast<CArchiveBranch*>( parent.internalPointer() );

	CArchiveBranch *branch = parentNode->branch( name );
	if ( branch )
		return createIndex( branch->row(), 0, branch );

	return QModelIndex();
};

QModelIndex CArchiveModel::parent( const QModelIndex &idx ) const {

	if ( not idx.isValid() )
		return QModelIndex();

	CArchiveBranch *branch = ( CArchiveBranch * )idx.internalPointer();
	CArchiveBranch *parentNode = branch->parent();

	if ( parentNode == tree )
		return QModelIndex();

	return createIndex( parentNode->row(), 0, parentNode );
};

bool CArchiveModel::hasBranches( const QModelIndex &idx ) const {

	if ( not idx.isValid() )
		return true;

	else {
		CArchiveBranch *branch = static_cast<CArchiveBranch*>( idx.internalPointer() );
		return ( branch->branchCount() > 0 );
	}
};

QString CArchiveModel::nodeName( const QModelIndex idx ) const {

	return idx.data( 0 ).toString();
};

QString CArchiveModel::nodePath( const QModelIndex idx ) const {

	if ( not idx.isValid() )
		return QString();

	QString path = idx.data().toString();
	QModelIndex par = parent( idx );

	while ( par.isValid() ) {
		path = par.data().toString() + "/" + path;
		par = parent( par );
	}

	return path;
};

void CArchiveModel::growTree() {

	beginResetModel();

	LibArchiveQt *archive = new LibArchiveQt( archiveName );

	tree = new CArchiveBranch( "Name", QIcon::fromTheme( "folder" ), 0 );

	Q_FOREACH( ArchiveEntry *ae, archive->listArchive() ) {
		QString name = QString( ae->name );
		int type = ae->type;
		QString size = CPrime::FileFunc::formatSize( ae->size );

		QStringList tokens = QStringList() << name.split( "/", Qt::SkipEmptyParts );
		QString iconName = ( ( type == AE_IFDIR ) ? "folder" : mimeDb.mimeTypeForFile( tokens.value( 0 ) ).iconName() );
		tree->addBranch( new CArchiveBranch( tokens.value( 0 ), QIcon::fromTheme( iconName ), ( ( type == AE_IFDIR ) ? "" : size ), tree ) );
		if ( tokens.size() == 1 )
			continue;

		CArchiveBranch *branch = tree;
		for( int i = 1; i < tokens.size(); i++ ) {
			branch = branch->branch( tokens.value( i - 1 ) );
			QString iconName = ( ( type == AE_IFDIR ) ? "folder" : mimeDb.mimeTypeForFile( tokens.value( i ) ).iconName() );
			branch->addBranch( new CArchiveBranch( tokens.value( i ), QIcon::fromTheme( iconName ), ( ( type == AE_IFDIR ) ? "" : size ), branch ) );
		}

		qApp->processEvents();
	}

	endResetModel();
	emit archiveLoaded();
};

/*
	*
	* CNewArchiveModel - For new archives
	*
*/

CNewArchiveModel::CNewArchiveModel() : QAbstractItemModel() {

	/* Root node */
	plantTree();
};

CNewArchiveModel::~CNewArchiveModel() {

};

int CNewArchiveModel::rowCount( const QModelIndex &parent ) const {

	CArchiveBranch *parentItem;
	if ( parent.column() > 0 )
		return 0;

	if ( not parent.isValid() )
		parentItem = tree;

	else
		parentItem = static_cast<CArchiveBranch*>( parent.internalPointer() );

	return parentItem->branchCount();
};

int CNewArchiveModel::columnCount( const QModelIndex & ) const {

	return 2;
};

Qt::ItemFlags CNewArchiveModel::flags( const QModelIndex &idx ) const {

	if ( not idx.isValid() )
		return Qt::NoItemFlags;

	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
};

QVariant CNewArchiveModel::data( const QModelIndex &index, int role ) const {

	if ( not index.isValid() )
		return QVariant();

	CArchiveBranch *node = static_cast<CArchiveBranch*>( index.internalPointer() );
	switch( role ) {

		case Qt::DisplayRole: {
			if ( index.column() == 0 )
				return node->name();

			else if ( index.column() == 1 )
				return node->size();

			return QVariant();
		}

		case Qt::DecorationRole: {
			if ( index.column() == 0 )
				return node->icon();

			return QVariant();
		}

		case Qt::TextAlignmentRole: {
			if ( index.column() == 0 )
				return ( 0x0001 | 0x0080 );

			else if ( index.column() == 1 )
				return ( 0x0002 | 0x0080 );

			else
				return Qt::AlignCenter;
		}

		case Qt::InitialSortOrderRole: {
			return Qt::AscendingOrder;
		}

		case Qt::AccessibleTextRole: {
			return node->name();
		}

		case Qt::ToolTipRole: {
			if ( index.column() == 0 )
				return node->data( Qt::UserRole + 1 );

			return QString();
		}

		case Qt::UserRole + 1: {

			return node->data( Qt::UserRole + 1 );
		}

		default: {

			return QVariant();
		}
	}
};

QVariant CNewArchiveModel::headerData( int section, Qt::Orientation orientation, int role ) const {

	if ( orientation != Qt::Horizontal )
		return QVariant();

	switch( section ) {
		case 0: {
			switch( role ) {
				case Qt::DisplayRole:
					return "Name";

				case Qt::TextAlignmentRole:
					return Qt::AlignCenter;

				default:
					return QVariant();
			}
		}

		case 1: {
			switch( role ) {
				case Qt::DisplayRole:
					return "Size";

				case Qt::TextAlignmentRole:
					return Qt::AlignCenter;

				default:
					return QVariant();
			}
		}

		default: {
			return QVariant();
		}
	}
};

QModelIndex CNewArchiveModel::index( int row, int column, const QModelIndex &parent ) const {

    if ( row < 0 or column < 0 )
        return QModelIndex();

    if ( ( row > rowCount( parent ) ) and ( column > columnCount( parent ) ) )
		return QModelIndex();

	CArchiveBranch *parentNode;
	if ( not parent.isValid() )
		parentNode = tree;

	else
		parentNode = (CArchiveBranch *)parent.internalPointer();

	CArchiveBranch *branch = parentNode->branch( row );
	if ( branch )
		return createIndex( row, column, branch );

	else
		return QModelIndex();
};

QModelIndex CNewArchiveModel::index( QString name, const QModelIndex &parent ) const {

	CArchiveBranch *parentNode;
	if ( not parent.isValid() )
		parentNode = tree;

	else
		parentNode = static_cast<CArchiveBranch*>( parent.internalPointer() );

	CArchiveBranch *branch = parentNode->branch( name );
	if ( branch )
		return createIndex( branch->row(), 0, branch );

	return QModelIndex();
};

QModelIndex CNewArchiveModel::parent( const QModelIndex &idx ) const {

	if ( not idx.isValid() )
		return QModelIndex();

	CArchiveBranch *branch = ( CArchiveBranch * )idx.internalPointer();
	CArchiveBranch *parentNode = branch->parent();

	if ( parentNode == tree )
		return QModelIndex();

	return createIndex( parentNode->row(), 0, parentNode );
};

QString CNewArchiveModel::nodeName( const QModelIndex idx ) const {

	return idx.data( 0 ).toString();
};

QString CNewArchiveModel::nodePath( const QModelIndex idx ) const {

	if ( not idx.isValid() )
		return QString();

	QString path = idx.data().toString();
	QModelIndex par = parent( idx );

	while ( par.isValid() ) {
		path = par.data().toString() + "/" + path;
		par = parent( par );
	}

	return path;
};

void CNewArchiveModel::growTree( QStringList files ) {

	beginResetModel();

	Q_FOREACH( QString file, files ) {
		QFileInfo fi( file );

		QString name = fi.filePath();
		QString size = ( fi.isDir() ? "" : CPrime::FileFunc::formatSize( fi.size() ) );
		QMimeType mime = mimeDb.mimeTypeForFile( name );

		QString iconName = mime.iconName();
		tree->addBranch( new CArchiveBranch( file, QIcon::fromTheme( iconName ), size, tree ) );
	}

	endResetModel();
};

void CNewArchiveModel::chopTree( QModelIndexList indexes ) {

	beginResetModel();

	Q_FOREACH( QModelIndex idx, indexes ) {
		CArchiveBranch *branch = (CArchiveBranch *)idx.internalPointer();
		qWarning() << branch->name();
		tree->removeBranch( branch );
	}

	endResetModel();
};

void CNewArchiveModel::plantTree() {

	tree = new CArchiveBranch( "Name", QIcon::fromTheme( "folder" ), 0 );
};
