/*
    *
    * This file is a part of CoreArchiver.
    * Archive manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/

#include <QFileDialog>
#include <QScroller>

#include <cprime/infofunc.h>
#include <cprime/filefunc.h>
#include <libarchiveqt.h>

#include "carchivemodel.hpp"
#include "corearchiver.h"
#include "ui_corearchiver.h"

/*
	*
	* Notes on UI
	*
	* I have made several changes to the UI to make it a bit more intuitive.
	* nameLE  -> CREATE:  Will show the full file path of the archive and formatsCB will be hidden.
	*            EXTRACT: Will show just the only file name (no suffix), and formatsCB will provide the suffix.
	* workDir -> CREATE:  Path where the archive will be saved
	*            EXTRACT: Destination folder where the archive will be extracted to
	* progressBar can show the progress of compressing and extracting archives. To be hidden when no work is being done.
	*
*/

corearchiver::corearchiver( QWidget *parent ) : QWidget( parent ), ui( new Ui::corearchiver ) {

    ui->setupUi(this);

    QPalette pltt(palette());
    pltt.setColor(QPalette::Base, Qt::transparent);
    setPalette(pltt);

    loadSettings();
    startSetup();
};

corearchiver::~corearchiver() {

    delete ui;
};

void corearchiver::startSetup() {

    QScroller::grabGesture( ui->archiveView, QScroller::LeftMouseButtonGesture );

	supportedFormats = QStringList() << ".zip" << ".tar.xz" << ".tar.gz" << ".tar.bz2" << ".7z" <<
                            ".tar" << ".iso" << ".cpio" << ".shar" << ".tar.Z" << ".ar" << ".pax";

    connect( ui->back, &QToolButton::clicked, [=](){ ui->page->setCurrentIndex( 0 ); } );


    /* Hide the progress bar */
    ui->progressBar->hide();

    this->resize(800, 500);

    // set window size
    if (uiMode != 0) {
        this->setWindowState(Qt::WindowMaximized);

        if (uiMode == 2){
            // all toolbuttons in icon mode in toolBar
            for (QToolButton *b : ui->toolBar->findChildren<QToolButton *>()) {
                if (b) {
                    b->setIconSize(toolBarIconSize);
                    b->setToolButtonStyle(Qt::ToolButtonIconOnly);
                }
            }
        }

    }

    ui->page->setCurrentIndex(0);

    ui->archiveView->setIconSize(listViewIconSize);

};

/**
 * @brief Loads application settings
 */
void corearchiver::loadSettings()
{
    uiMode = sm->value("CoreApps", "UIMode");
    toolBarIconSize = sm->value("CoreApps", "ToolBarIconSize");
    listViewIconSize = sm->value("CoreApps", "ListViewIconSize");
}


void corearchiver::on_arcAction_clicked() {

    if ( ui->arcAction->text() == "Compress" ) {
        QString archive = QDir( ui->locationLE->text() ).filePath( ui->nameLE->text() + ui->formatsCB->currentText() );
        if (QFile(archive).exists()) {
            CPrime::InfoFunc::messageEngine("CoreArchiver", "corearchiver", "Cannot create archive file exists", "A file with that name already exists.\nPlease give a new name.", this);
            return;
        }

		compress();
    } else
		extract();
};

void corearchiver::compress() {

	/* The archive will always be created at the current location. */
	QString archive = QDir( ui->locationLE->text() ).filePath( ui->nameLE->text() + ui->formatsCB->currentText() );

	/* LibArchiveQt object to create the archive */
	LibArchiveQt *arc = new LibArchiveQt( archive );
	connect( arc, &LibArchiveQt::progress, ui->progressBar, &QProgressBar::setValue );

    QObject::connect( arc, &LibArchiveQt::jobComplete, this, &corearchiver::showArchiveMessage );

	/* Update the input files from the model */
	CNewArchiveModel *newModel = qobject_cast<CNewArchiveModel *>( ui->archiveView->model() );
	QStringList inputList;
	for ( int i = 0; i < newModel->rowCount(); i++ )
		inputList << newModel->index( i, 0 ).data().toString();

	/* Use the path common to all the input files as the relative path */
	arc->updateInputFiles( inputList, LibArchiveQt::CommonRelativePath );

	/* ProgressBar will have a range of 0 -> 100 during extraction */
	ui->progressBar->setRange( 0, 100 );
	ui->progressBar->show();

	/* Disable the UI */
	setDisabled( true );

	/* Start the archive extraction */
    arc->createArchive();
};

void corearchiver::extract() {

    const QString destP = ui->locationLE->text();

    LibArchiveQt *arc = new LibArchiveQt( ui->nameLE->text() );
	connect( arc, &LibArchiveQt::progress, ui->progressBar, &QProgressBar::setValue );

	arc->setDestination( destP );				// LibArchiveQt will create destP if it does not exist
    QObject::connect( arc, &LibArchiveQt::jobComplete, this, &corearchiver::showExtractMessage );

	/* ProgressBar will have a range of 0 -> 100 during extraction */
	ui->progressBar->setRange( 0, 100 );
	ui->progressBar->show();

	/* Disable the UI */
	setDisabled( true );

	/* Start the archive extraction */
    arc->extractArchive();
};

void corearchiver::sendFiles(const QStringList &paths) {

    if ( !paths.count() ) {
        ui->page->setCurrentIndex( 0 );
        return;
    }

	ui->page->setCurrentIndex( 1 );

    QStringList supported;
    supported << "zip" << "tar.xz" << "tar.gz" << "tar.bz2" << "7z" <<
                     "tar" << "iso" << "cpio" << "shar" << "tar.Z" << "ar" << "xar";

    QFileInfo fi(paths.at(0));
    QString suffix = fi.suffix();

    // extrate
    if (supported.contains(suffix, Qt::CaseInsensitive)){
        Q_FOREACH( QString path, paths ) {
            QFileInfo fi( path );
            if ( fi.exists() ) {
                loadArchive( fi.absoluteFilePath() );
                break;
            }
        }

    }
    // create
    else {
        loadFiles(paths);
    }

};

void corearchiver::showArchiveMessage() {

	ui->progressBar->hide();

    CPrime::InfoFunc::messageEngine( "CoreArchiver", "corearchiver", "Archived Successfully", "No issues, everthing went smoothly", this );

	/* Re-enable the UI */
	setEnabled( true );

	/* Open the just created archive for viewing */
	loadArchive( QDir( ui->locationLE->text() ).filePath( ui->nameLE->text() + ui->formatsCB->currentText() ) );
};

void corearchiver::showExtractMessage() {

    ui->progressBar->hide();

    CPrime::InfoFunc::messageEngine( "CoreArchiver", "corearchiver", "Extracted Successfully...", "No issues, everthing went smoothly", this );

	/* Re-enable the UI */
	setEnabled( true );
};

void corearchiver::on_locationTB_clicked() {

	QString location = QFileDialog::getExistingDirectory(
		this,
		"CoreArchiver | Set extraction location",
		QDir::homePath()
	);

	if ( not location.isEmpty() )
		ui->locationLE->setText( location );
};

void corearchiver::loadArchive( QString fileName ) {

	/* We will put the full archive path in nameLE, and make it read only */
    ui->nameLE->setText( fileName );
	ui->nameLE->setReadOnly( true );

	/* Work dir: Show the current directory */
	ui->locationLE->setText( QDir::currentPath() );

	/* Hide the formats combo; no use in viewing the archive */
	ui->formatsCB->hide();

	/* Add/Remove files buttons need to be hidden */
	ui->addFiles->hide();
    ui->removeFile->hide();

	/* arcAction button text */
	ui->arcAction->setText( "Extract" );
    ui->arcAction->setIcon(QIcon::fromTheme("archive-extract"));

	/* Show the progressBar */
	ui->progressBar->show();
	ui->progressBar->setRange( 0, 0 );

	/* Setup the view and load the archive */
	CArchiveModel *model = new CArchiveModel();
	connect( model, &CArchiveModel::archiveLoaded, ui->progressBar, &QProgressBar::hide );

	ui->archiveView->setModel( model );
	model->loadArchive( fileName );

	ui->archiveView->header()->setSectionResizeMode( 0, QHeaderView::Stretch );
	ui->archiveView->header()->setSectionResizeMode( 1, QHeaderView::Fixed );
    ui->archiveView->header()->resizeSection( 1, 100 );
};

void corearchiver::loadFiles(const QStringList files){

    /* Set the path in locationLE */
    ui->locationLE->setText( QDir::currentPath() );

    ui->nameLE->setText( "Archive" );

    /* arcAction button text */
    ui->arcAction->setText( "Compress" );

    /* show the formats combo */
    ui->formatsCB->show();
    ui->formatsCB->setCurrentIndex( 0 );

    /* Add/Remove files buttons need to be shown */
    ui->addFiles->show();
    ui->removeFile->show();

    /* Hide the progressBar */
    ui->progressBar->hide();

    /* Setup the view and load the archive */
    CNewArchiveModel *model = new CNewArchiveModel();
    ui->archiveView->setModel( model );

    ui->archiveView->header()->setSectionResizeMode( 0, QHeaderView::Stretch );
    ui->archiveView->header()->setSectionResizeMode( 1, QHeaderView::Fixed );
    ui->archiveView->header()->resizeSection( 1, 200 );

    model->growTree(files);
}
void corearchiver::on_openArchive_clicked() {

	ui->page->setCurrentIndex( 1 );

	/* Get our archive name */
	QString filterList = "Archives (*" + supportedFormats.join( " *" ) + ");;All files (*.*)";

	QString fileName = QFileDialog::getOpenFileName(
		this,
		"CoreArchiver | Open archive",
		QDir::homePath(),
		filterList
	);

	/* If the user has not selected any archive, return to welcome page */
	if ( fileName.isEmpty() ) {
		ui->page->setCurrentIndex( 0 );
		return;
	}

	loadArchive( fileName );
};

void corearchiver::on_createArchive_clicked() {

    ui->page->setCurrentIndex( 1 );

	/* We will put the full archive path in nameLE, and make it read only */
	ui->nameLE->setText( "Archive" );
	ui->nameLE->setReadOnly( false );

	/* Set the path in locationLE */
	ui->locationLE->setText( QDir::currentPath() );

    /* show the formats combo */
	ui->formatsCB->show();
	ui->formatsCB->setCurrentIndex( 0 );

	/* Add/Remove files buttons need to be shown */
	ui->addFiles->show();
    ui->removeFile->show();

	/* arcAction button text */
	ui->arcAction->setText( "Compress" );
    ui->arcAction->setIcon(QIcon::fromTheme("archive-insert"));
	ui->arcAction->setDisabled( true );			// No files to compress

	/* Hide the progressBar */
	ui->progressBar->hide();

	/* Setup the view and load the archive */
	CNewArchiveModel *model = new CNewArchiveModel();
	ui->archiveView->setModel( model );

	ui->archiveView->header()->setSectionResizeMode( 0, QHeaderView::Stretch );
	ui->archiveView->header()->setSectionResizeMode( 1, QHeaderView::Fixed );
	ui->archiveView->header()->resizeSection( 1, 200 );

    // Open the Add file Dialog
    on_addFiles_clicked();
};

void corearchiver::on_addFiles_clicked() {

    QStringList list = QFileDialog::getOpenFileNames( this, "Add files", QDir::homePath(), "*" );

	/* Our view model */
	CNewArchiveModel *newModel = qobject_cast<CNewArchiveModel *>( ui->archiveView->model() );

	/* Start adding the files to the archive */
	newModel->growTree( list );

	/* If there are files in the archive, enable the compress button */
	if ( newModel->rowCount() )
		ui->arcAction->setEnabled( true );
}

void corearchiver::on_removeFile_clicked()
{
    QModelIndexList indexList = ui->archiveView->selectionModel()->selectedRows();

    /* Our view model */
    CNewArchiveModel *newModel = qobject_cast<CNewArchiveModel *>( ui->archiveView->model() );
    newModel->chopTree( indexList );

    /* If there are no files in the archive, disable the compress button */
    if ( newModel->rowCount() <= 0)
        ui->arcAction->setDisabled( true );
}
